/*

Tyre creation macro example and testbed.
Tested with Megapov 0.4, 0.5 and 0.5a
Runs in about 2 hours on a pII/450

fd 02 jul 2000

*/
#include "colors.inc"
#include "CVTF_EX2.inc"
//#version unofficial MegaPov 0.4;

// Smoothed angular sector. Already used in the gear mess package

#macro gear_cut(N, rmin, rmax, delta, rconge)
  #if (N*(delta+2*rconge) > 2*pi*rmin)
    #debug "Doesn't fit!\n"
  #else
    union{
      difference{
	cylinder {-y, y, rmax-rconge}
	cylinder {-1.1*y, 1.1*y, rmin+rconge}
	plane {-x, delta/2 rotate 180/N*y}
	plane {x, delta/2 rotate -180/N*y}
      }
      #local a = 180/pi*asin((rconge+delta/2)/(rconge+rmin));
      #local b = 180/pi*asin((rconge+delta/2)/(rmax-rconge));
      difference{
	cylinder {-y, y, rmin+rconge}
	cylinder {-1.1*y, 1.1*y, rmin}
	plane {-x, 0 rotate (180/N-a)*y}
	plane {x, 0 rotate -(180/N-a)*y}
      }
      difference{
	cylinder {-y, y, rmax}
	cylinder {-1.1*y, 1.1*y, rmax-rconge}
	plane {-x, 0 rotate (180/N-b)*y}
	plane {x, 0 rotate -(180/N-b)*y}
      }
      cylinder {-y*1.0001, y*1.0001, rconge translate <0, 0, rmin+rconge> rotate (180/N-a)*y}
      cylinder {-y*1.0001, y*1.0001, rconge translate <0, 0, rmin+rconge> rotate -(180/N-a)*y}
      cylinder {-y*1.0001, y*1.0001, rconge translate <0, 0, rmax-rconge> rotate (180/N-b)*y}
      cylinder {-y*1.0001, y*1.0001, rconge translate <0, 0, rmax-rconge> rotate -(180/N-b)*y}
      bounded_by {cylinder {-y, y, rmax-rconge}}
    }
  #end
#end

// Rib

#declare Jante=object{
  union{
    difference{
      cylinder {-6*y, 6*y, 30*.5}
      cylinder {-7*y, -5*y, 14}
      cylinder {7*y, -7*y, 13}
    }
    difference{
      sphere {0, 1 scale <15, 4, 15>}
      sphere {0, 1 scale <15, 4, 15> translate 1*y}
      #local NN=7;
      #local i=0;
      #while (i < NN)
	object{gear_cut(NN, 7, 10, 3, 1) scale <1, 10, 1> rotate 360/NN*i*y}
	#local i = i+1;
      #end
      translate 3*y
    }
    cylinder {3*y, -6*y, 3}
    cylinder {-6*y, -7*y, 4}
    sphere {0, 1 scale <2, 1, 2> translate -7*y}
    #local i = 0;
    #while (i < 8)
      sphere {<3.5, -7, 0>, .3 rotate 360*i/8*y}
      #local i = i+1;
    #end
  }
  pigment {rgb <1, 1, .4>}
  normal {crackle bump_size .4}
  finish {phong 0.2 phong_size 3}
  scale 1/3
}

#declare Pneu=union{
  object{
    Tyre(
      8,   // R
      5,   // Rint
      4,   // Ep
      "CVTF_IM1.png",
      2,   // Em
      1,   // Hbase
      0.5, // Htop
      0.1  //k_lathe
    )
    pigment {White*.1}               // White*.3
    finish {phong 0.2 phong_size 3}  // phong 0.4
  }
  object{Jante}
}

/*object{
  Pneu
  rotate -90*z translate -25*x
}
*/
