// This file is licensed under the terms of the CC-LGPL. 
#include "MPCM_AU2.inc"

#macro MPCM_AuxiliarBezier( PuntoA, PuntoB, ArregloBezier, I ) 
  #local Origen         = < PuntoA.x, PuntoA.y >;
  #local Final          = < PuntoB.x, PuntoB.y >;
  #local Recorre        = Final-Origen;
  #if ( I > 0 )        
    #local ArregloBezier[ I-1 ]         = Origen+vdot( PuntoA, z )*Recorre/3;
  #end  // #if ( I < 0 )
  #local ArregloBezier[ I ]             = Origen+Recorre*vdot( PuntoA, z )/3;
  #local ArregloBezier[ I+1 ]           = ( Origen+Final )/2;
  #local ArregloBezier[ I+2 ]           = ArregloBezier[ I+1 ];
  #local ArregloBezier[ I+3 ]           = Final-vdot( PuntoB, z )*Recorre/3;
  #local ArregloBezier[ I+4 ]           = ArregloBezier[ I+3 ];
  #local ArregloBezier[ I+5 ]           = Final;
  #local ArregloBezier[ I+6 ]           = Final; 
#end  // #macro MPCM_AuxiliarBezier     

#macro MPCM_AuxiliarBezierMedio( PuntoA, PuntoB, ArregloBezier, I ) 
  #if ( I > 0 )        
    #local ArregloBezier[ I-1 ]         = ( PuntoA+PuntoB )/2;
  #end  // #if ( I < 0 )
  #local ArregloBezier[ I ]             = ( PuntoA+PuntoB )/2;
  #local ArregloBezier[ I+1 ]           = PuntoB;
  #local ArregloBezier[ I+2 ]           = PuntoB;
#end  // #macro MPCM_AuxiliarBezierMedio

#macro MPCM_RellenarArregloBezier( ArregloPrevio, ArregloBezier )
  #local TotalPuntos                    = dimension_size( ArregloPrevio, 1 );
  #local C                              = 0;
  #while ( C < TotalPuntos )        
    #local PuntoA                       = ArregloPrevio[ C ];
    #if ( C = ( TotalPuntos-1 ) )         
      #local PuntoB                     = ArregloPrevio[ 0 ];
    #else  // #else ( C = ( TotalPuntos-1 ) )                
      #local PuntoB                     = ArregloPrevio[ C+1 ];
    #end   // #else ( C = ( TotalPuntos-1 ) )              
    MPCM_AuxiliarBezier( PuntoA, PuntoB, ArregloBezier, 8*C )
    #local C                            = C+1;
  #end  // #while ( C < TotalPuntos )                       
  #local ArregloBezier[ 8*C-1 ]         = ArregloBezier[ 0 ];
#end  // #macro MPCM_RellenarArregloBezier  
   
#macro MPCM_AuxiliarBezierAbierto( PuntoA, PuntoB, ArregloBezier, I, FactorB ) 
  #local Origen         = PuntoA;
  #local Final          = PuntoB;
  #local Recorre        = Final-Origen;
  #if ( I > 0 )        
    #local ArregloBezier[ I-1 ]         = Origen+FactorB*Recorre/3;
  #end  // #if ( I < 0 )
  #local ArregloBezier[ I ]             = Origen+Recorre*FactorB/3;
  #local ArregloBezier[ I+1 ]           = ( Origen+Final )/2;
  #local ArregloBezier[ I+2 ]           = ArregloBezier[ I+1 ];
  #local ArregloBezier[ I+3 ]           = Final-FactorB*Recorre/3;
  #local ArregloBezier[ I+4 ]           = ArregloBezier[ I+3 ];
  #local ArregloBezier[ I+5 ]           = Final;
  #local ArregloBezier[ I+6 ]           = Final; 
#end  // #macro MPCM_AuxiliarBezierAbierto     
   
#macro MPCM_RellenarArregloBezierAbierto( ArregloPrevio, ArregloBezier, FactorB )
  #local TotalPuntos                    = dimension_size( ArregloPrevio, 1 )-1;
  #local C                              = 0;
  #while ( C < TotalPuntos )        
    #local PuntoA                       = ArregloPrevio[ C ];
    #local PuntoB                       = ArregloPrevio[ C+1 ];
    MPCM_AuxiliarBezierAbierto( PuntoA, PuntoB, ArregloBezier, 8*C, FactorB )
    #local C                            = C+1;
  #end  // #while ( C < TotalPuntos )                               
  #local ArregloBezier[ 0 ]             = ArregloPrevio[ 0 ];
  #local ArregloBezier[ 8*TotalPuntos-2 ]         = ArregloBezier[ 8*TotalPuntos-4 ];
  #local ArregloBezier[ 8*TotalPuntos-1 ]         = ArregloPrevio[ TotalPuntos ];
#end  // #macro MPCM_RellenarArregloBezierAbierto         
 
#macro MPCM_RellenarArregloBezierCerrado( ArregloPrevio, ArregloBezier, FactorB )
  #local TotalPuntos                    = dimension_size( ArregloPrevio, 1 );
  #local C                              = 0;
  #while ( C < TotalPuntos )        
    #local PuntoA                       = ArregloPrevio[ C ];    
    #if ( C = ( TotalPuntos-1 ) )      
      #local PuntoB                     = ArregloPrevio[ 0 ];
    #else  // #else ( C = ( TotalPuntos-1 ) )
      #local PuntoB                     = ArregloPrevio[ C+1 ];  
    #end   // #else ( C = ( TotalPuntos-1 ) )
    MPCM_AuxiliarBezierAbierto( PuntoA, PuntoB, ArregloBezier, 8*C, FactorB )
    #local C                            = C+1;
  #end  // #while ( C < TotalPuntos )                               
  #local ArregloBezier[ 8*C-1 ]         = ArregloBezier[ 0 ];
#end  // #macro MPCM_RellenarArregloBezierCerrado         
 
#macro MPCM_RellenarArregloBezierMedio( ArregloPrevio, ArregloBezier )
  #local TotalPuntos                    = dimension_size( ArregloPrevio, 1 );
  #local C                              = 0;
  #while ( C < TotalPuntos )        
    #local PuntoA                       = ArregloPrevio[ C ];
    #if ( C = ( TotalPuntos-1 ) )         
      #local PuntoB                     = ArregloPrevio[ 0 ];
    #else  // #else ( C = ( TotalPuntos-1 ) )                
      #local PuntoB                     = ArregloPrevio[ C+1 ];
    #end   // #else ( C = ( TotalPuntos-1 ) )              
    MPCM_AuxiliarBezierMedio( PuntoA, PuntoB, ArregloBezier, 4*C )
    #local C                            = C+1;
  #end  // #while ( C < TotalPuntos )                       
  #local ArregloBezier[ 4*C-1 ]         = ArregloBezier[ 0 ];
#end  // #macro MPCM_RellenarArregloBezierMedio 

#macro MPCM_PrismaBezier( ArregloPrevio, BanderaSweep, Y0, Y1 ) 
  prism                                 {    
    #local TotalPuntos                  = dimension_size( ArregloPrevio, 1 ); 
    #local TotalBezier                  = 8*TotalPuntos; 
    #declare MPCM_ArregloBezier         = array [ TotalBezier ]
    MPCM_RellenarArregloBezier( ArregloPrevio, MPCM_ArregloBezier )
    #if ( BanderaSweep )   
      conic_sweep
    #else  // #else ( BanderaSweep )
      linear_sweep
    #end   // #else ( BanderaSweep ) 
    bezier_spline 
    Y0, Y1, TotalBezier,
    #local C                            = 0;
    #while ( C < TotalBezier )        
      MPCM_ArregloBezier[ C ]
      #local C                          = C+1;
    #end  // #while ( C < TotalBezier )   
  }  // prism
#end  // #macro MPCM_PrismaBezier   

#macro MPCM_PrismaBezierMedio( ArregloPrevio, BanderaSweep, Y0, Y1 ) 
  prism                                 {    
    #local TotalPuntos                  = dimension_size( ArregloPrevio, 1 ); 
    #local TotalBezier                  = 4*TotalPuntos; 
    #declare MPCM_ArregloBezier         = array [ TotalBezier ]
    MPCM_RellenarArregloBezierMedio( ArregloPrevio, MPCM_ArregloBezier )
    #if ( BanderaSweep )   
      conic_sweep
    #else  // #else ( BanderaSweep )
      linear_sweep
    #end   // #else ( BanderaSweep ) 
    bezier_spline 
    Y0, Y1, TotalBezier,
    #local C                            = 0;
    #while ( C < TotalBezier )        
      MPCM_ArregloBezier[ C ]
      #local C                          = C+1;
    #end  // #while ( C < TotalBezier )   
  }  // prism
#end  // #macro MPCM_PrismaBezierMedio  

#macro MPCM_SweepBezierAbierto( ArregloPrevio, Radio, FactorB )  
  #if ( MPCM_POV3_1 )  
    union                               {
      #local TotPuntos                  = dimension_size( ArregloPrevio, 1 );  
      #local ArregloPivote              = array [ 8*( TotPuntos- 1 ) ]
      #local TotCurv                    = 2*( TotPuntos-1 );
      MPCM_RellenarArregloBezierAbierto( ArregloPrevio, ArregloPivote, FactorB )   
      #local C                          = 0;
      #while ( C < TotCurv )                  
        #local PuntoA                   = ArregloPivote[ 4*C ];
        #local PuntoB                   = 
           0.35*( ArregloPivote[ 4*C ]+ArregloPivote[ 4*C+3 ] )+0.3*ArregloPivote[ 4*C+1 ];
        #local PuntoC                   = ArregloPivote[ 4*C+3 ];
        MPCM_SweepTriplePunto( PuntoA, PuntoB, PuntoC, Radio, true )
        #local C                        = C+1;
      #end  // #while ( C < TotCurv )
    } 
  #else   
    sphere_sweep                        {         
      #local TotPuntos                  = dimension_size( ArregloPrevio, 1 );  
      #local ArregloPivote              = array [ 8*( TotPuntos- 1 ) ];      
      #local TotCurv                    = 2*( TotPuntos-1 );
      MPCM_RellenarArregloBezierAbierto( ArregloPrevio, ArregloPivote, FactorB )
      cubic_spline                                                            
      ( TotCurv )*2+3,
      2*ArregloPivote[ 0 ]-ArregloPivote[ 1 ], Radio,
      #local C                          = 0;
      #while ( C < TotCurv )         
        ArregloPivote[ 4*C ], Radio,
        0.35*( ArregloPivote[ 4*C ]+ArregloPivote[ 4*C+3 ] )+0.3*ArregloPivote[ 4*C+1 ], Radio
        #local C                        = C+1;
      #end  // #while ( C < TotCur )    
      ArregloPivote[ 8*( TotPuntos-1 )-1 ], Radio,
      2*ArregloPivote[ 8*( TotPuntos-1 )-1 ]-ArregloPivote[ 8*( TotPuntos-1 )-2 ], Radio   
    }
  #end
#end  // macro MPCM_SweepBezierAbierto       

#macro MPCM_SweepBezierCerrado( ArregloPrevio, Radio, FactorB )  
  #if ( MPCM_POV3_1 )  
    union                               {
      #local TotPuntos                  = dimension_size( ArregloPrevio, 1 );  
      #local ArregloPivote              = array [ 8*TotPuntos ]
      #local TotCurv                    = 2*TotPuntos;
      MPCM_RellenarArregloBezierCerrado( ArregloPrevio, ArregloPivote, FactorB )   
      #local C                          = 0;
      #while ( C < TotCurv )                  
        #local PuntoA                   = ArregloPivote[ 4*C ];
        #local PuntoB                   = 
           0.35*( ArregloPivote[ 4*C ]+ArregloPivote[ 4*C+3 ] )+0.3*ArregloPivote[ 4*C+1 ];
        #local PuntoC                   = ArregloPivote[ 4*C+3 ];
        MPCM_SweepTriplePunto( PuntoA, PuntoB, PuntoC, Radio, true )
        #local C                        = C+1;
      #end  // #while ( C < TotCurv )
    } 
  #else   
    sphere_sweep                        {         
      #local TotPuntos                  = dimension_size( ArregloPrevio, 1 );  
      #local ArregloPivote              = array [ 8*TotPuntos ];      
      #local TotCurv                    = 2*TotPuntos;
      MPCM_RellenarArregloBezierCerrado( ArregloPrevio, ArregloPivote, FactorB )
      cubic_spline                                                            
      ( TotCurv )*2+3,          
      #local C                          = TotCurv-1;
      0.35*( ArregloPivote[ 4*C ]+ArregloPivote[ 4*C+3 ] )+0.3*ArregloPivote[ 4*C+1 ], Radio
      #local C                          = 0;
      #while ( C < TotCurv )         
        ArregloPivote[ 4*C ], Radio,
        0.35*( ArregloPivote[ 4*C ]+ArregloPivote[ 4*C+3 ] )+0.3*ArregloPivote[ 4*C+1 ], Radio
        #local C                        = C+1;
      #end  // #while ( C < TotCur )    
      ArregloPivote[ 8*TotPuntos-1 ], Radio,        
      #local C                          = 0;
      0.35*( ArregloPivote[ 4*C ]+ArregloPivote[ 4*C+3 ] )+0.3*ArregloPivote[ 4*C+1 ], Radio
    }
  #end
#end  // macro MPCM_SweepBezierCerrado 