// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com Escazu, San Jose,  
// Apdo. 964-1250 Es Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 22 enero 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// PBTT PalitaBasuraTodoTipo
// El archivo dibuja una palita para basura de uso domestico.
// Es muy poco flexible, pero incluye una macro para modificar 
// algunas de sus caracteristicas principales.
// Es compatible con POVRay 3.1.
//
// La macro principal no tiene parametros. 
// #macro PBTT_PalitaBasura()    
//
// La siguiente macro permite un mayor control de las caracteristicas.
// #macro PBTT_PalitaBasuraTodoTipo(
//        AltoPalita, Abertura, AberturaPosterior, Lateral,
//        GruesoPlastico, AltoTotal, DiametroPalo,
//        TexturaPalita, TexturaMadera )
//      AltoPalita              : Referencia para el alto de la palita contenedora
//          de desechos. Valor por defecto 54*L.
//      Abertura                : Abertura frontal en X de la pala.
//          Valor por defecto 200*L.
//      AberturaPosterior       : La abertura posterior debe ser menor a la frontal.
//          Valor por defecto 143*L.                             
//      Lateral                 : Referencia para la medida lateral en Z de la pala.
//          Valor por defecto 168*L.
//      GruesoPlastico          : Grosor del plastico o material de la palita.
//          Valor por defecto 1.5*L.
//      AltoTotal               : Referencia para el alto total de la palita con la madera.
//          Valor por defecto 575*L.
//      DiametroPalo            : Diametro del palo que soporta la palita.
//          Valor por defecto 17*L.
//      TexturaPalita           : Textura principal de la palita.
//          Valor por defecto PBTT_TexturaPalita.
//      TexturaMadera           : Textura del palo soporte.
//          Valor por defecto PBTT_TexturaMadera.
//
// La palita se encuentra aproximadamente contenido en las siguientes coordenadas.
//      Minimo  = < -100,    0,    0 >*L
//      Maximo  = <  100,  590,  175 >*L
// El eje de la palita se encuentra inclinado 5 grados hacia adelante. 
// El frente mira hacia Z-, note que el aparato se encuentra completamente
// contenido en el hemisferio Z+.
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare PBTT_DefaultTexturaPalita      
//      Textura principal de la palita.
// #declare PBTT_DefaultTexturaMadera            
//      Textura del palo soporte.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare PBTT_TexturaPalita                  = texture { PBTT_DefaultTexturaPalita }        
//      Textura principal del pico.
// #declare PBTT_TexturaMadera                  = texture { PBTT_DefaultTexturaMadera }
//      Textura del soporte de madera.
//
// Para colocarlo simplemente
// #include "PBTT.inc"
// object        { PBTT_PalitaBasura( ) }
//                                 , 
// Ejemplo con la macro detallada, mismo objeto default
// #include "PBTT.inc"   
// object                       {
//   PBTT_PalitaBasuraTodoTipo( 0.054, 0.200, 0.143, 0.168, 0.0015, 0.575, 0.017,
//        PBTT_TexturaPalita, PBTT_TexturaMadera )
// }
//        
// Con modificadores
// #include "PBTT.inc"
// #include "textures.inc"
// #declare PBTT_TexturaPalita       = PinkAlabaster  
// object        { PBTT_PalitaBasura( ) }
// 

#include "woods.inc"   
#include "textures.inc"

#declare PBTT_L                 = 0.001;

#declare PBTT_DefaultTexturaPalita              =
  texture                       { 
    average
    texture_map                 {  
      [ 0.75 Polished_Chrome ]
      [ 0.25 Rust ]
    }                    
    /*
    pigment                     {
      color Brown
    }
    normal                      {
      waves
    } */
  }

#ifndef ( PBTT_TexturaPalita )
  #declare PBTT_TexturaPalita                   = PBTT_DefaultTexturaPalita
#end  // #ifndef ( PBTT_TexturaPalita )

#declare PBTT_DefaultTexturaMadera              =
  texture                       {
    T_Wood2
    normal                      {
      agate
    }
  }       

#ifndef ( PBTT_TexturaMadera )
  #declare PBTT_TexturaMadera                   = PBTT_DefaultTexturaMadera
#end  // #ifndef ( PBTT_TexturaMadera )

#macro PBTT_PalitaBasuraTodoTipo(
        AltoPalita, Abertura, AberturaPosterior, Lateral,
        GruesoPlastico, AltoTotal, DiametroPalo,
        TexturaPalita, TexturaMadera )
  union                         {
    #local EscalaBasica         =
                < AberturaPosterior/2, AltoPalita, Lateral >;
    #local AnguloPalaD          = 5;
    #local AnguloPala           = radians( AnguloPalaD );
    #local EscalaInterior       =
                EscalaBasica*<1,2,0.75*cos( AnguloPala ) >;
                
    // Formas Basicas
    #local FormaBasica          =
      difference                {
        intersection            {
          union                   {
            box                   {
              -1, 1
              matrix              <1,0,0,
                                   0,1,0,
                                   AberturaPosterior/Abertura-1,0,1,
                                   0,0,0>
            }
            box                   {
              -1, 1
              rotate              90*y
              matrix              <1,0,0,
                                   0,1,0,
                                   1-AberturaPosterior/Abertura,0,1,
                                   0,0,0>
            }

          }
          superellipsoid          {
            <0.8,0.1>
            rotate              90*y
            scale               <2,1,1>
          }
        }
        plane                   {
          y, 0
        }
        plane                   {
          -z, 0
        }
      }
    #local FormaBasicaEscalada  =
      object                    {
        FormaBasica
          scale                 EscalaBasica
      }
    #local EnvolventeInterior   =
      prism                     {
        conic_sweep
        1,
        1+0.5*Lateral*sin( AnguloPala )/AltoPalita,
        5,
        <-0.7,+1>,
        <-Abertura/AberturaPosterior+0.4, -0.1>,
        <+Abertura/AberturaPosterior-0.4, -0.1>,
        <+0.7,+1>,
        <-0.7,+1>
        translate       -y
      }

    // Cuerpo Girado AnguloPala
    difference                  {
      union                       {
        // Palo madera
        union                       {
          cylinder                    {
            AltoPalita/2*y, ( AltoTotal-DiametroPalo/2 )*y, DiametroPalo/2
          }
          sphere                    {
            ( AltoTotal-DiametroPalo/2 )*y, DiametroPalo/2
          }
          texture                   {
            TexturaMadera
          }
        }
        // Palita
        union                       {
          // Soporte Palo
          difference                {
            cylinder                    {
              0, AltoPalita*y,
              DiametroPalo/2+GruesoPlastico
            }
            cylinder                    {
              AltoPalita/2*y, ( AltoTotal-DiametroPalo/2 )*y, DiametroPalo/2
            }
          }
          // Envolvente Lateral
          union                          {
            difference                     {
              union                         {
                object                        {
                  FormaBasicaEscalada
                }
                difference                  {
                  object                    {
                    FormaBasica
                    scale             EscalaBasica+DiametroPalo/4*<1,0,1/2>
                    translate               +DiametroPalo/4*z
                  }
                  object                    {
                    FormaBasica
                    scale           EscalaBasica-GruesoPlastico*(y+z)+
                                            +DiametroPalo/4*<1,0,1/2>*1.001
                    translate               +DiametroPalo/4*z*1.001
                    translate               -GruesoPlastico*y
                  }
                }
              }
              object                        {
                FormaBasica
                scale               EscalaBasica-GruesoPlastico*<1,1,1>
                translate           GruesoPlastico*( y-z )
              }
            }
          }
          texture                   {
            TexturaPalita
          }
        }
        translate                   +Lateral*z
        rotate                      -AnguloPalaD*x
      }
      object                    {
        EnvolventeInterior
        scale           EscalaInterior
      }
    }
    // Base Pala
    union                       {
      difference                  {
        object                    {
          EnvolventeInterior
          scale           EscalaInterior
        }
        object                    {
          EnvolventeInterior
          scale           EscalaInterior-GruesoPlastico
          translate       GruesoPlastico*y
        }
        plane                   {
          z, 0
        }
        plane                     {
          -y, 0
          translate               +GruesoPlastico*y
          rotate                  -AnguloPalaD*x
        }
      }
      difference                 {
        cylinder                {
          -x, x, 1
        }
        #local C                = 0;
        #local TotalCilindros   = 12;
        #while ( C < TotalCilindros )
          cylinder                {
            -z, z, 1
            scale                 <0.7/TotalCilindros,0.5,1>
            translate             y
            translate             ( -1+2*( C+0.5 )/TotalCilindros )*x
          }
          #local C              = C+1;
        #end
        plane                   {
          y, 0
        }
        scale                 EscalaInterior*<1.1,0.3*Lateral*sin( AnguloPala )/AltoPalita,0.3>
        translate             +0.35*EscalaInterior.z*z
      }
      texture                   {
        TexturaPalita
      }
    }
  }
#end  // #macro PBTT_PalitaBasuraTodoTipo

#macro PBTT_PalitaBasura()
  object                        {
    #local L                    = PBTT_L;
    #local AltoPalita           = 54*L;
    #local Abertura             = 200*L;
    #local AberturaPosterior    = 143*L;
    #local Lateral              = 168*L;
    #local GruesoPlastico       = 1.5*L;
    #local AltoTotal            = 575*L;
    #local DiametroPalo         = 17*L;
    PBTT_PalitaBasuraTodoTipo(
        AltoPalita, Abertura, AberturaPosterior, Lateral,
        GruesoPlastico, AltoTotal, DiametroPalo,
        PBTT_TexturaPalita, PBTT_TexturaMadera )
  }
#end  // #macro PPTT_PalitaBasura
