// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho...  16 junio 2015
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// SPRD SemaforoPeatonalRojoDetener
// El archivo dibuja un semaforo para cruce peatonal corriente.
// Es muy poco flexible, pero incluye una macro para modificar 
// algunas de sus caracteristicas principales.
// Es compatible con POVRay 3.1.
//
// La macro principal tiene cuatro parametros. 
// #macro SPRD_Semaforo( IzquierdoDerecho, BanderaRed,
//                        BanderaGreen, AnguloSemaforo )
//      IzquierdoDerecho        : "true" dibuja el semaforo con la caja hacia X-,
//              "false" hacia X+.                                               
//      BanderaRed              : "true" enciende la luz superior roja.
//      BanderaGreen            : "true" encienda la luz inferior verde.
//      AnguloSemaforo          : Angulo en grados de giro de la caja de luz.
//
// La siguiente macro permite un mayor control de las caracteristicas.
// #macro SPRD_SemaforoPeatonalRojoDetener(
//          IzquierdoDerecho, BanderaRed, BanderaGreen, AnguloSemaforo,
//          AltoFundamental, GruesoLateral, CajaLuz, AltoBoton,
//          TexturaPrincipal, TexturaOscura, TexturaTornillo, TexturaBoton )
//      IzquierdoDerecho        : "true" dibuja el semaforo con la caja hacia X-,
//              "false" hacia X+.                                               
//      BanderaRed              : "true" enciende la luz superior roja.
//      BanderaGreen            : "true" encienda la luz inferior verde.
//      AnguloSemaforo          : Angulo en grados de giro de la caja de luz.
//      AltoFundamental         : Alto del semaforo. Valor por  defecto 1975*L. 
//      GruesoLateral           : Grueso lateral del poste de semaforo. 
//              Valor por defecto 50*L.
//      CajaLuz                 : Vector 3D para dimensionar la caja de luces del semaforo.
//              Valor por defecto < 280, 420, 100 >*L.                           
//      AltoBoton               : Altura del boton para cambiar de luz.
//              Valor por defecto 800*L.
//      TexturaPrincipal        : Textura principal a aplicar al semaforo.
//              Valor por defecto SPRD_TexturaPrincipal.
//      TexturaOscura           : Textura para la caja oscura de luces.  
//              Las luces se ven claramente gracias al contraste del fondo.
//              Valor por defecto SPRD_TexturaOscura. 
//      TexturaTornillo         : Textura a aplicar a los tornillos.
//              Valor por defecto SPRD_TexturaTornillo.
//      TexturaBoton            : Textura a aplicar al boton.
//              Valor por defecto SPRD_TexturaBoton.
//      
// El semaforo con parametros <true,true,false,0> tiene las siguientes dimensiones.
//      Minimo  = < -390,    0, -190 >*L
//      Maximo  = <    0, 1975,   50 >*L 
// El parametro IzquierdoDerecho produce un resultado parecido a un scale <-1,1,1>;
// pero el angulo de rotacion es independiente de este parametro.
// El semaforo se encuentra completamente a un lado del plano YZ, y sobre el
// plano XZ. El lateral del semaforo partido a la mitad por el eje Y+.
// Las luces miran hacia Z-, con gravedad Y-.
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare SPRD_DefaultTexturaPrincipal  
//      Textura principal del semaforo.
// #declare SPRD_DefaultTexturaOscura      
//      Textura para la caja oscura de luces. Las luces se ven claramente
//      gracias al contraste con el fondo.
// #declare SPRD_DefaultTexturaTornillo           
//      Textura para los tornillos.
// #declare SPRD_DefaultTexturaBoton       
//      Textura plastica para el boton.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare SPRD_TexturaPrincipal               = texture { ... }                 
//      Textura principal del semaforo.
// #declare SPRD_TexturaOscura                  = texture { ... }
//      Textura para la caja oscura de luces. Las luces se ven claramente
//      gracias al contraste por el fondo.                                   
// #declare SPRD_TexturaTornillo                = texture { ... }
//      Textura para los tornillos.
// #declare SPRD_TexturaBoton                   = texture { ... }   
//      Textura plastica para el boton.
// #declare SPRD_PigmentRojo                    = Red;                      
//      Pigmento para la luz roja encendida. Se ve por contraste con el fondo.
// #declare SPRD_PigmentRojoApagado             = color Red transmit 0.85;   
//      Pigmento para la luz roja apagada.
// #declare SPRD_PigmentVerde                   = Green;
//      Pigmento para la luz verde encendida. Se aprecia por contraste con el fondo.
// #declare SPRD_PigmentVerdeApagado            = color Green transmit 0.95;        
//      Pigmento para la luz verde apagada.
//
// Para colocarlo simplemente
// #include "SPRD.inc"
// object        { SPRD_Semaforo( true, true, false, 5 ) }
//                                 , 
// Ejemplo con la macro detallada, mismo objeto default
// #include "SPRD.inc"   
// object                       {
//   SPRD_SemaforoPeatonalRojoDetener( true, true, false, 5,
//          1.975, 0.050, <0.280,0.420,0.100>, 0.800,    
//          SPRD_TexturaPrincipal, SPRD_TexturaOscura,
//          SPRD_TexturaTornillo, SPRD_TexturaBoton )
// }
//        
// Con modificadores
// #include "SPRD.inc"
// #include "textures.inc"
// #declare SPRD_TexturaPrincipal   = PinkAlabaster
// object        { SPRD_Semaforo( true, true, false, 5 ) }
// 
#include "colors.inc"

#declare SPRD_L                         = 0.001;  

#declare SPRD_PigmentRojo               = Red;
#declare SPRD_PigmentVerde              = Green;
#declare SPRD_PigmentRojoApagado        = color Red transmit 0.85;
#declare SPRD_PigmentVerdeApagado       = color Green transmit 0.95;

#declare SPRD_DefaultTexturaPrincipal   =
  texture                               {
    pigment                             {
      bumps
      pigment_map                       {
        [ 0.0 Yellow ]
        [ 1.0 Yellow*0.25 ]
      }
      ramp_wave
    } 
    normal                              {   
      bumps
    }
  }

#ifndef ( SPRD_TexturaPrincipal )
  #declare SPRD_TexturaPrincipal        = SPRD_DefaultTexturaPrincipal
#end  // #ifndef ( SPRD_TexturaPrincipal )

#declare SPRD_DefaultTexturaOscura      =
  texture                               {
    pigment                             {
      bumps
      pigment_map                       {
        [ 0.0 Gray10 ]
        [ 1.0 Gray20 ]
      }
      ramp_wave
    }
  }

#ifndef ( SPRD_TexturaOscura )
  #declare SPRD_TexturaOscura           = SPRD_DefaultTexturaOscura
#end  // #ifndef ( SPRD_TexturaOscura )

#declare SPRD_DefaultTexturaBoton       =
  texture                               {
    pigment                             {
      Red
    }
  }

#ifndef ( SPRD_TexturaBoton )
  #declare SPRD_TexturaBoton            = SPRD_DefaultTexturaBoton
#end  // #ifndef ( SPRD_TexturaBoton )

#declare SPRD_DefaultTexturaTornillo    =
  texture                               {
    pigment                             {
      radial
      frequency                         4
      pigment_map                       {
        [ 0.0 Silver ]
        [ 0.8 Silver ]
        [ 0.85 Black ]
        [ 0.9 Silver ]
        [ 1.0 Silver ]
      }
    }
    normal                               {
      radial
      frequency                         8
    }
  }

#ifndef ( SPRD_TexturaTornillo )
  #declare SPRD_TexturaTornillo         = SPRD_DefaultTexturaTornillo
#end  // #ifndef ( SPRD_TexturaTornillo )

#macro SPRD_SemaforoPeatonalRojoDetener(
          IzquierdoDerecho, BanderaRed, BanderaGreen, AnguloSemaforo,
          AltoFundamental, GruesoLateral, CajaLuz, AltoBoton,
          TexturaPrincipal, TexturaOscura, TexturaTornillo, TexturaBoton )
  union                                 {
    #if ( IzquierdoDerecho )
      #local SignoIzquierdo                      = -1;
    #else
      #local SignoIzquierdo                      = +1;
    #end
    #local AltoSoporteCajaLuz           = 140*CajaLuz.y/420;
    #local GruesoBracito                = 20*CajaLuz.y/420;
    #local FactorBracito                = 0.75;
    #local RadioLuz                     =
                0.4*min( CajaLuz.x, CajaLuz.y/2 );
    #local FactorLuz                    = 1.40;
    // Mayor a uno el FactorLuz
    #local MaxYCajaLuz                  =
                AltoFundamental-( AltoSoporteCajaLuz+GruesoBracito )/2;
    // Poste Soporte
    box                                 {
      < -2*GruesoLateral/4, 0,  -GruesoLateral/2 >,
      < +GruesoLateral/4, AltoFundamental, +GruesoLateral/2 >
      scale                     < SignoIzquierdo, 1, 1 >
    }
    // SoporteCajaLuz
    #local C                            = 0;
    #while ( C < 2 )
      union                               {
        box                             {
          < 0, -AltoSoporteCajaLuz/2, -0.9*GruesoLateral/2 >,
          < SignoIzquierdo*GruesoLateral/20, AltoSoporteCajaLuz/2,
                        0.9*GruesoLateral/2 >
        }
        box                             {
          < 0, GruesoBracito/2, -0.9*GruesoLateral/2 >,
          < SignoIzquierdo*FactorBracito*CajaLuz.x, -GruesoBracito/2,
                        0.9*GruesoLateral/2 >
        }
        cylinder                        {
          GruesoBracito/2*y, -GruesoBracito/2*y,
          0.9*GruesoLateral/2
          translate     SignoIzquierdo*FactorBracito*CajaLuz.x*x
        }
        sphere                          {
          0, GruesoLateral/6
          scale         <1,0.5,1>
          texture                       {
            TexturaTornillo
            rotate                      90*x
          }
          translate     +GruesoBracito/2*y
          translate     SignoIzquierdo*FactorBracito*CajaLuz.x*x
          #if ( C = 1 )
            scale                       <1,-1,1>
          #end // #if ( C = 1 )
        }
        sphere                          {
          0, 0.20*GruesoLateral/2
          scale         <0.5,1,1>
          texture                       {
            TexturaTornillo
          }
          translate             0.30*AltoSoporteCajaLuz*y
          translate                     SignoIzquierdo*GruesoLateral*x/20
        }
        sphere                          {
          0, 0.20*GruesoLateral/2
          scale         <0.5,1,1>
          texture                       {
            TexturaTornillo
          }
          translate            -0.30*AltoSoporteCajaLuz*y
          translate                     SignoIzquierdo*GruesoLateral*x/20
        }
        translate                       ( MaxYCajaLuz+GruesoBracito/2 )*y
        translate                       SignoIzquierdo*( GruesoLateral/4 )*x
        translate                       -( CajaLuz.y+GruesoBracito )*C*y
      }
      #local C                          = C+1;
    #end  // #while ( C < 2 )
    // Boton
    union                               {
      cylinder                          {
        -GruesoLateral*x/4, 0.8*GruesoLateral/2*x, 1.1*GruesoLateral/2
      }
      cylinder                          {
        0.9*GruesoLateral/2*x, 0.7*GruesoLateral/2*x, 0.9*GruesoLateral/2
      }
      difference                        {
        cylinder                          {
          1.0*GruesoLateral/2*x, 0.875*GruesoLateral/2*x, 1.1*GruesoLateral/2
        }
        cylinder                          {
          1.1*GruesoLateral/2*x, 0.925*GruesoLateral/2*x, 0.35*GruesoLateral/2
          translate             0.35*GruesoLateral/2*y
          texture { TexturaBoton }
        }        
      }
      difference                        {
        cylinder                          {
          1.1*GruesoLateral/2*x, 0.925*GruesoLateral/2*x, 0.45*GruesoLateral/2
        }        
        cylinder                          {
          1.2*GruesoLateral/2*x, 0.915*GruesoLateral/2*x, 0.35*GruesoLateral/2
        }
        plane                           {
          y, 0
        }
        translate             0.35*GruesoLateral/2*y
      }
      #local C                          = 1;
      #while ( C < 5 )
        sphere                          {
          0, 0.10*GruesoLateral/2
          scale         <0.5,1,1>
          texture                       {
            TexturaTornillo
          }
          translate                     1.0*GruesoLateral/2*x
          translate                     0.90*GruesoLateral/2*y
          rotate                        C*360/5*x
        }
        #local C                        = C+1;
      #end  // #while (  C < 5 )
      translate                         GruesoLateral.x/2*x
      scale                             < SignoIzquierdo, 1, 1 >
      translate                         AltoBoton*y
    }
    // Semaforo Verdadero
    union                               {
      difference                        {
        #local FormaBase                =
           prism                             {
             -1, 1,
             11,
             < -0.90, -1.00 >,
             < -0.90, -0.90 >,
             < -0.85, -0.90 >,
             < -1.00, -0.50 >,
             < -0.30, +1.00 >,
             < +0.30, +1.00 >,
             < +1.00, -0.50 >,
             < +0.85, -0.90 >,
             < +0.90, -0.90 >,
             < +0.90, -1.00 >,
             < -0.90, -1.00 >
           }
        object                          {
          FormaBase
          scale                           CajaLuz/2
        }
        torus                           {
          1.1*RadioLuz, 0.05*RadioLuz
          rotate                        90*x
          translate                     +CajaLuz.y*y/4
          translate                     -CajaLuz.z*z/2
        }
        torus                           {
          1.1*RadioLuz, 0.05*RadioLuz
          rotate                        90*x
          translate                     -CajaLuz.y*y/4
          translate                     -CajaLuz.z*z/2
        }
        #local C                        = 0;
        #while ( C < 2 )
          difference                    {
            object                      {
              FormaBase
              scale                     <0.9,1.025,0.9>*CajaLuz/2
              translate                 ( 2*C-1 )*CajaLuz.y*y
            }
            cylinder                    {
              -y, y, 0.25
              scale                     <0.9,1.050,0.9>*CajaLuz/2
            }
            #local D                    = 0;
            #while ( D < 8 )
              box                       {
                <-2,-2,-0.05>,
                <+2,+2,+0.05>
                rotate                  180*D*y/8
                scale                     <0.9,1.050,0.9>*CajaLuz/2
              }
              #local D                  = D+1;
            #end
          }
          #local C                      = C+1;
        #end
        box                             {
          < -1, -0.05,-0.895>,
          < +1, +0.05,-1.10 >
          scale                           CajaLuz/2
        }
      }
      // Luz Arriba C=0 y Abajo C=1
      #local C                          = 0;
      #while ( C < 2 )
        union                           {
            union                       {
              sphere                    {
                0, 0.8*RadioLuz
                scale           <1,1,0.1>
                translate       -CajaLuz.z*z/2
                #if ( C = 0 )
                  #if ( BanderaRed )
                    pigment { SPRD_PigmentRojo }
                    finish                  {
                      ambient 0.8
                      diffuse 0.2
                    }
                  #else
                    texture { TexturaOscura }
                    texture { pigment { SPRD_PigmentRojoApagado } }
                  #end
                #else
                  #if ( BanderaGreen )
                    pigment { SPRD_PigmentVerde }
                    finish                  {
                      ambient 0.8
                      diffuse 0.2
                    }
                  #else
                    texture { TexturaOscura }
                    texture { pigment { SPRD_PigmentVerdeApagado } }
                  #end
                #end
              }
              difference                  {  
                #local RadioExLuz         = max( RadioLuz, sqrt( vdot( CajaLuz, CajaLuz ) ) );
                sphere                    {
                  0, 0.825*RadioLuz
                  scale           <1,1,0.1>
                  translate       -CajaLuz.z*z/2
                  texture { TexturaOscura }
                }
                #local D                = 0;
                #local TotalD           = 10;
                #while ( D < TotalD )
                  #local E              = 0;
                  #while ( E < TotalD )
                    cylinder            {
                      -RadioExLuz*z, RadioExLuz*z, 0.070*RadioLuz
                      translate    ( TotalD-1-2*D )/( TotalD-1 )*RadioLuz*0.7*y
                      #if ( mod( E, 2 ) = 0 )
                        translate   -1/( TotalD-1 )*RadioLuz*0.7*y
                      #end
                      translate    ( TotalD-1-2*E )/( TotalD-1 )*RadioLuz*0.7*x
                    }
                    #local E            = E+1;
                  #end  // #while ( E < 3 )
                  #local D              = D+1;
                #end  // #while ( D < 3 )
              }
            }    
            
          difference                    {
            union                       {
              intersection                  {
                cylinder                    {
                  0, -FactorLuz*CajaLuz.z*z, RadioLuz
                }
                cylinder                    {
                  -CajaLuz.x*x, +CajaLuz.x*x, RadioLuz
                  scale        < 1, 1, 0.5*CajaLuz.z/RadioLuz >
                  translate                 +RadioLuz*y
                  translate                 -( FactorLuz-0.5 )*CajaLuz.z*z
                }
              }
              difference                    {
                cylinder                    {
                  0, -( FactorLuz-0.5 )*CajaLuz.z*z, RadioLuz
                }
                cylinder                    {
                  -CajaLuz.x*x, +CajaLuz.x*x, RadioLuz
                  scale        < 1, 1, min( RadioLuz, 0.5*CajaLuz.z )/RadioLuz >
                  translate                 -RadioLuz*y
                  translate                 -( FactorLuz-0.5 )*CajaLuz.z*z
                }
              }
            }  // union para la capa negra de luz 
            cylinder                    {
              0, -FactorLuz*CajaLuz.z*z, 0.975*RadioLuz
              translate  -0.01*CajaLuz.z*z
            }                            
            cylinder                    {
              FactorLuz*CajaLuz.z*z, -FactorLuz*CajaLuz.z*z, 0.8*RadioLuz
              translate  -0.01*CajaLuz.z*z
            }                            
            translate         -CajaLuz.z*z/2
            texture                     {
              TexturaOscura
            }
          }
          
          // Tornillo
          #local D                      = 0;
          #while ( D < 4 )
            sphere                      {
              0, RadioLuz*0.075
              scale             <1,1,0.5>
              translate         RadioLuz*1.05*( x+y )
              rotate                    90*D*z
              texture                   {
                TexturaTornillo
              }
              translate         -CajaLuz.z*z/2
            }
            cylinder                    {
              -0.10*y, 0.10*y, 0.050
              translate                 <1,0.35,-1>*0.9
              #if ( mod( D, 2 ) = 1 )
                scale                   <-1,1,1>
              #end
              #if ( div( D, 2 ) = 1 )
                scale                   <1,-1,1>
              #end
              scale                     CajaLuz/2
              texture   { TexturaOscura }
            }
            #local D                    = D+1;
          #end  // #while ( D < 1 )
          translate                     ( 0.5-C )*CajaLuz.y*y/2
        }
        #local C                        = C+1;
      #end  // #while ( C < 2 )
      rotate                            AnguloSemaforo*y
      translate     SignoIzquierdo*FactorBracito*CajaLuz.x*x
      translate                         ( MaxYCajaLuz-CajaLuz.y/2 )*y
      translate                         SignoIzquierdo*( GruesoLateral/4 )*x
    } 
    translate                           +SignoIzquierdo*GruesoLateral/2*x
    texture                             {
      TexturaPrincipal
    }
  }
#end  // #macro SPRD_SemaforoPeatonalRojoDetener

#macro SPRD_Semaforo( IzquierdoDerecho, BanderaRed,
                        BanderaGreen, AnguloSemaforo )
  object                                {
    #local L                            = SPRD_L;
    #local AltoFundamental              = 1975*L;
    #local GruesoLateral                =  50*L;
    #local CajaLuz                      = < 280, 420, 100 >*L;
    #local AltoBoton                    = 800*L;
    SPRD_SemaforoPeatonalRojoDetener
        ( IzquierdoDerecho, BanderaRed,
          BanderaGreen, AnguloSemaforo,
          AltoFundamental, GruesoLateral, CajaLuz, AltoBoton,
          SPRD_TexturaPrincipal, SPRD_TexturaOscura,
          SPRD_TexturaTornillo, SPRD_TexturaBoton )
  }
#end  // #macro SPRD_Semaforo()
